#if UNITY_EDITOR && UNITY_IOS
using UnityEngine;
using UnityEditor;
using System.IO;
using UnityEditor.Callbacks;
using UnityEditor.iOS.Xcode;
using System.Collections.Generic;
using System.Linq;
 
public class AppGuardPostProcessBuild : MonoBehaviour
{
    [PostProcessBuild]
    public static void OnPostprocessBuild(BuildTarget buildTarget, string path)
    {
        if (buildTarget == BuildTarget.iOS)
        {
            string projPath = PBXProject.GetPBXProjectPath(path);
            PBXProject pBXProject = new PBXProject();
 
            pBXProject.ReadFromString(File.ReadAllText(projPath));
            string targetGuid = pBXProject.GetUnityMainTargetGuid();
            string projectGuid = pBXProject.ProjectGuid();
            /**
             * AppGuard for iOS SDK 빌드에 필요한 빌드 옵션을 추가 합니다. ( 필수 )
             */
            pBXProject.AddBuildProperty(projectGuid, "OTHER_LDFLAGS", "-lz");

            /**
             * bitcode build 옵션을 설정 합니다. ( 선택 )
             */
            pBXProject.SetBuildProperty(projectGuid, "ENABLE_BITCODE", "NO");
 
            /**
             * AppGuard for iOS SDK 구동에 필요한 환경 파일을 추가 합니다. ( 필수 )
             */
            var fileName = "appguard";
            var filePath = Path.Combine("Assets/", fileName);
            if(File.Exists(Path.Combine(path, fileName)) == false)
            {
                File.Copy(filePath, Path.Combine(path, fileName));
                pBXProject.AddFileToBuild(targetGuid, pBXProject.AddFile(fileName, fileName, PBXSourceTree.Source));
            }
            

            fileName = "appguard106000";
            filePath = Path.Combine("Assets/", fileName);
            if(File.Exists(Path.Combine(path, fileName)) == false)
            {
                File.Copy(filePath, Path.Combine(path, fileName));
                pBXProject.AddFileToBuild(targetGuid, pBXProject.AddFile(fileName, fileName, PBXSourceTree.Source));
            }
            
            fileName = "appguard.mf";
            filePath = Path.Combine("Assets/", fileName);
            if(File.Exists(Path.Combine(path, fileName)) == false)
            {
                File.Copy(filePath, Path.Combine(path, fileName));
                pBXProject.AddFileToBuild(targetGuid, pBXProject.AddFile(fileName, fileName, PBXSourceTree.Source));
            }
            
            fileName = "appguard.crt";
            filePath = Path.Combine("Assets/", fileName);
            if(File.Exists(Path.Combine(path, fileName)) == false)
            {
                File.Copy(filePath, Path.Combine(path, fileName));
                pBXProject.AddFileToBuild(targetGuid, pBXProject.AddFile(fileName, fileName, PBXSourceTree.Source));
            }
            
            File.WriteAllText(projPath, pBXProject.WriteToString());
 
        }
    }
 
}
#endif