using UnityEngine;
using System; 
using System.Collections;
using System.Runtime.InteropServices;
public class GameGuardEventType {
	public class S2Auth {
		public const int S2AUTH_RESULT_SUCCESS				= 1;
		public const int S2AUTH_RESULT_RETRY				= 2;
		public const int S2AUTH_RESULT_FAIL 				= 3;
	}

	public class Detect {
		public const int DETECT_RUNNING_BAD_APPLICATION 	= 21;
		/*
			... 
			Please refer to the AppGuard API documentation.
		*/
	}
}

public class AppGuardUnityManager : MonoBehaviour {
	private static AppGuardUnityManager _instance;

	#if UNITY_ANDROID && !UNITY_EDITOR
		private AndroidJavaObject mAppGuardClientUnityBridge = null;
	#endif

	#if UNITY_IPHONE && !UNITY_EDITOR
		[DllImport("__Internal")]
		public static extern void AppGuardInit();
		[DllImport("__Internal")]
		public static extern string AppGuardGetUDID();

		[DllImport("__Internal")]
		public static extern void AppGuardSetUserID(string userid);

		[DllImport("__Internal")]
		public static extern void AppGuardSetCertificationID(string userid, long retryTimeSec);
		
		[DllImport("__Internal")]
		public static extern void AppGuardSetReserved1();

		[DllImport("__Internal")]
		public static extern void AppGuardStartCustomMacroMode();
		
		[DllImport("__Internal")]
		public static extern void AppGuardStopCustomMacroMode();
		
		[DllImport("__Internal")]
		public static extern void AppGuardSetCustomMacroRepeatEnable(bool bEnable);
		
		[DllImport("__Internal")]
		public static extern void AppGuardSetCustomMacroRepeatCount(int repeat);
		
		[DllImport("__Internal")]
		public static extern void AppGuardSetCustomMacroDensityEnable(bool bEnable);

		[DllImport("__Internal")]
		public static extern void AppGuardSetCustomMacroDensityRatio(int ratio);
		
		[DllImport("__Internal")]
		public static extern void AppGuardSetCustomMacroDensityConditionCount(int count);
	
		[DllImport("__Internal")]
		public static extern void AppGuardSetCustomMacroDensityConditionItemCount(int count);
	#endif

	/**
     * 서버 인증 관련 이벤트 Callback을 수신 받는 메서드로 서버 인증이 진행되면 AppGuard는 해당 메소드를 호출하여 상태 정보를 전달합니다.
	 *
	 * 만약, 서버인증 결과에 대한 클라이언트단의 추가적인 처리가 필요하지 않다면 빈(empty) 메서드로 구현하여 주시기를 권장합니다.
	 * ex) public void onS2AuthTryCallback(string data) { ; };
     *
     * @param   data   서버 인증 시도에 대한 상태값과 필요 시 추가적인 결과값이 ','구분자를 이용하여 통합되어 전달됩니다.
	 *                 해당 값은 String.Split(',')등의 메서드를 사용하여 데이터를 구분하여야합니다.
	 *                 [1,-] String형으로 변환된 상태값으로 int.Parse()등의 메서드로 int형으로 변환 후 사용됩니다.
	 *                 [-,2] 상태값이 S2AUTH_RESULT_SUCCESS일 경우 추가적으로 서버인증에 사용한 UniqueClientID가
	 *                      추가적으로 포함됩니다.
     */
	public void onS2AuthTryCallback(string data) {
		string[] args = data.Split(',');
		
		switch (int.Parse(args[0])) {
		case GameGuardEventType.S2Auth.S2AUTH_RESULT_SUCCESS:
			/*
			서버 인증이 성공하였으며, 정상적으로 인증이 완료되었습니다.
			*/
			Debug.Log ("S2AUTH_RESULT_SUCCESS(UniqueID: " + args[1] + ")");
			break;
		case GameGuardEventType.S2Auth.S2AUTH_RESULT_RETRY:
			/*
			서버 인증이 실패하였으며, 재 인증을 시도할 것입니다. 일시적인 클라이언트 네트워크 장애 또는 서버 장애 일 수 있으며, 해당 재시도는
			내부 메커니즘에 따라 최대 3분 동안 수행될 수 있습니다.
			*/
			Debug.Log ("S2AUTH_RESULT_RETRY");
			break;
		case GameGuardEventType.S2Auth.S2AUTH_RESULT_FAIL:
			/*
			서버 인증이 완전히 실패하였으며, 더 이상 서버 인증을 시도하지 않습니다.
			*/
			Debug.Log ("S2AUTH_RESULT_FAIL");
			break;
		}
	}
	
	/**
     * 보안 정책 위반 탐지 관련 이벤트 Callback을 수신 받는 메서드로 정책 위반이 탐지되면 AppGuard는 해당 메소드를 호출하여 상태 정보를 전달합니다.
	 *
	 * 만약, [Android]의 경우 빌드 시 Appguarding 옵션에서 'Disable displaying security violation message'를 선택하지 않았다면
	 * 해당 메서드를 빈(empty) 메서드로 구현하여 주시기를 권장합니다.
	 * ex) public void onViolationCallback(string data) { ; };
     *
     * @param   data   보안 정책 위반 탐지에 대한 상태값을 전달합니다.
	 *                 인자인 data 변수의 값은 보안정책위반 상태를 의미하는 값이 string 형태로 변환되어 전달된 값으로 반드시 int.Parse()등을 
	 *                 이용하여 int 형으로 변환하여 사용해야 합니다. 변환된 값은 보안정책위반값과 조치사항에 대한 상태값(음수=reported/양수=terminated)
     *                 을 함께 포함하고 있음으로 다음과 같은 코드를 사용하여 분리하여야 합니다.
	 *                 즉, 주어진 인자값을 int형으로 변환 후 절대값으로 변환한 값이 정책위반 코드(Value)값이 되며, 음수(x<0)의 경우 reported 상태,
     *                 양수(x>0)의 경우 terminated 상태를 의미합니다.
     */
	public void onViolationCallback(string data) {
		bool killed = (int.Parse(data) > 0);
		
		/*
		해당 data 인자 값이 양수를 가지면 설정된 정책 설정값에 따라 AppGuard는 약 30초 후 강제 종료를 수행합니다.
		그럼으로 사용자에게 앱이 종료될 것이라는 메시지를 해당 위치에서 노출하는 코드가 구현되어야합니다.
				
		[Android]의 경우 빌드 시 Appguarding 옵션에서 'Disable displaying security violation message'를
		체크하지 않았다면 특별한 메시지 출력 처리가 필요하지 않습니다.
		*/
		if(killed) {
			int code = Mathf.Abs(int.Parse(data));
			//... 이곳에 종료 메세지 처리를 위한 코드를 추가 해 줍니다.
		} 

	}

	/**
     * 해당 메소드는 AppGuard와 통신을 위한 앱의 콜백 함수를 앱가드에  등록 하는 메소드 입니다.
	 *
	 * 일반적으로 Unity 프로젝트의 첫번째 장면(Scene) 클래스의 Awake() 메서드에서 호출되하는 것을 권장하며 호출 방법은 다음과 같습니다.
	 * ex) AppGuardUnityManager.Instance.start();
	 */
    public void start() {
    	#if UNITY_IPHONE && !UNITY_EDITOR
    		AppGuardInit();
    	#endif

    }

    /**
     * AppGaurd for iOS framework는 실행중인 앱의 소유자를 구분하기 위하여 유일한 값이 UDID를 생성 합니다.
     * 해당 메소드는 AppGaurd for iOS framework에서 생성한 UDID값을 얻어 내기 위해 제공되는 메소드 입니다.
	 *
	 * 호출 방법은 다음과 같습니다.
	 * ex) AppGuardUnityManager.Instance.getAppGuardUdid();
	 */
	public string getAppGuardUdid() {
		string udid = "Not supported on this platform";
		#if UNITY_IPHONE && !UNITY_EDITOR
			udid = AppGuardGetUDID ();
		#endif
		return udid;
	}

	/**
     * 해당 메소드는 AppGuard에 보안 정책 위반 탐지 시 로그 서버에 함께 전달할 사용자 식별 ID를 지정합니다.
	 *
	 * 일반적인 호출 방법은 다음과 같습니다.
	 * ex) AppGuardUnityManager.Instance.setUserId([UserID]);
	 *
	 * @param   userid   사용자 식별 ID값
	 */
	public void setUserId(string userid) {
		#if UNITY_IPHONE && !UNITY_EDITOR
			AppGuardSetUserID (userid);
		#endif

		#if UNITY_ANDROID && !UNITY_EDITOR
			if ( mAppGuardClientUnityBridge == null ) {
				mAppGuardClientUnityBridge = new AndroidJavaObject ("com.inca.security.AppGuard.AppGuardClientUnityBridge");
			}
			mAppGuardClientUnityBridge.Call ("setUserId", userid);
		#endif
	}
	
	/**
     * 해당 메소드는 AppGuard에 서버 인증에 사용한 UniqueClientId를 지정하며 서버 인증을 시작하도록 지시합니다.
	 * 
	 * 만약, 이전 서버 인증이 완료되기 전에 해당 메소드가 다시 호출되면 기존 인증을 파기하고 새로 서버 인증을 시도합니다.
	 *
	 * 일반적인 호출 방법은 다음과 같습니다.
	 * ex) AppGuardUnityManager.Instance.setUniqueClientId([UserID], 180);
	 *
	 * @param   uniqueId       서버 인증에 사용한 UniqueClientID값
	 *          retryTimeSec   서버 인증을 시도할 최대 Timeout값, 기본값은 180(=3분)
	 */	
	public void setUniqueClientId(string uniqueId, long retryTimeSec) {
		#if UNITY_IPHONE && !UNITY_EDITOR
			AppGuardSetCertificationID (uniqueId, (int)retryTimeSec);
		#endif

		#if UNITY_ANDROID && !UNITY_EDITOR
			if ( mAppGuardClientUnityBridge == null ) {
				mAppGuardClientUnityBridge = new AndroidJavaObject ("com.inca.security.AppGuard.AppGuardClientUnityBridge");
			}

			mAppGuardClientUnityBridge.Call ("setUniqueClientID", uniqueId, (int)retryTimeSec);
		#endif
	}

	/**
     * 해당 메소드는 AppGuard의 매크로 탐지 이력을 초기화 하고 재 탐지를 수행하기 위한 함수 입니다.
	 * 
	 * 일반적인 호출 방법은 다음과 같습니다.
	 * ex) AppGuardUnityManager.Instance.setReserved1();
	 *
	 */	
	public void setReserved1() {
		#if UNITY_IPHONE && !UNITY_EDITOR
			AppGuardSetReserved1();
		#endif
	}	

	
	/**
     * 해당 메소드는 AppGuard의 Macro Customize RuleSet을 반영하기 위한 기능으로 게임 시작시 1회 호출해야 하며,
	 * startCustomMacroMode 메소드를 호출해야만 해당 RuleSet으로 동작합니다.
	 *
	 * 해당 기능을 이용할 경우, MacroRule value는 내부 코드에서와 같이 명시적으로 호출해야 합니다.
	 *
	 * 해당 옵션을 적용할 특정 Scene에서 호출하며, 일반적인 호출 방법은 다음과 같습니다.
	 * ex) AppGuardUnityManager.Instance.setCustomMacroRule();
	 *
	 */
	public void setCustomMacroRule() {
		#if UNITY_IPHONE && !UNITY_EDITOR
			/**
				* MacroRule의 value는 반드시 아래 코드와 같이 명시적으로 호출 해주어야 반영됩니다.
				* 만약 명시적으로 호출하지 않을시에는 Default value를 사용합니다.
				*
				* Repeat : 동일한 좌표의 패턴이 중복되는 경우 탐지
				*		AppGuardSetCustomMacroRepeatEnable : 
				* 			@param {boolean}		enable		해당 기능 사용여부 / Default : true
				* 		AppGuardSetCustomMacroRepeatCount	:
				*			@param {int}			count		N(count)개의 패턴이 연속적일때 탐지 / Default : 5
				*
				*
				* Density : 동일 좌표의 반복 클릭 상태값을 활용하는 방식으로, N개의 데이터에서 특정 클릭 갯수를 수렴할 경우 
				*			 수렴된 해당 데이터의 좌표들이 N개의 데이터에 존재하는 비율로 탐지
				* 		AppGuardSetCustomMacroRepeatEnable:
				* 			@param {boolean}		enable		해당 기능 사용여부 / Default : true
				* 		AppGuardSetCustomMacroDensityRatio:
				* 			@param {int}			ratio		탐지를 위한 N개의 데이터에 존재하는 비율	/ Default : 5
				* 		AppGuardSetCustomMacroDensityCoditionCount:
				* 			@param {int}			count		한개의 좌표가 클릭된 횟수				/ Default : 10
				* 		AppGuardSetCustomMacroDensityConditionItemCount:
				* 			@param {int}			count		한개의 좌표가 클릭된 횟수에 만족하는 좌표의 갯수	/ Default : 10
				*
				*/
			AppGuardSetCustomMacroRepeatEnable(true);
			AppGuardSetCustomMacroRepeatCount(5);

			AppGuardSetCustomMacroDensityEnable(true);
			AppGuardSetCustomMacroDensityRatio(5);
			AppGuardSetCustomMacroDensityConditionCount(10);
			AppGuardSetCustomMacroDensityConditionItemCount(10);
		#endif

		#if UNITY_ANDROID && !UNITY_EDITOR
			if ( mAppGuardClientUnityBridge == null ) {
				mAppGuardClientUnityBridge = new AndroidJavaObject ("com.inca.security.AppGuard.AppGuardClientUnityBridge");
			}

			/**
			* MacroRule의 value는 반드시 아래 코드와 같이 명시적으로 호출 해주어야 반영됩니다.
			* 만약 명시적으로 호출하지 않을시에는 Default value를 사용합니다.
			*
			* Repeat : 동일한 좌표의 패턴이 중복되는 경우 탐지
			*		setRepeatEnable : 
			* 			@param {boolean}		enable		해당 기능 사용여부 / Default : true
			* 		setRepeatCount	:
			*			@param {int}			count		N(count)개의 패턴이 연속적일때 탐지 / Default : 32
			*
			* EventTime : Touch Event time을 기준으로 탐지
			* 		setEventTimeEnable:
			* 			@param {boolean}		enable		해당 기능 사용여부 / Default : true
			* 		setEventTimeSamplingCount:
			* 			@param {int}			count		N(count)개의 패턴의 평균 Touch event time	/ Default : 12
			* 		setEventTimeThresholdRange:
			* 			@param {int}			min			탐지를 위한 Touch event time의 minimum value	/ Default : 0
			* 									max			탐지를 위한 Touch event time의 maximum value	/ Default : 8
			*
			* Density : 시간과 좌표를 둘다 활용하는 방식으로, N개의 데이터에서 시간값이 특정 시간대에 수렴할 경우 
			*			 수렴된 해당 데이터의 좌표들이 N개의 데이터에 존재하는 비율로 탐지
			* 		setDensityEnable:
			* 			@param {boolean}		enable		해당 기능 사용여부 / Default : true
			* 		setDensityRatio:
			* 			@param {int}			ratio		탐지를 위한 N개의 데이터에 존재하는 비율	/ Default : 5
			*
			*/
			AndroidJavaObject macroRule = new AndroidJavaObject ("com.inca.security.Macro.MacroRule");
			if ( macroRule != null ) {
				macroRule.Call ("setRepeatEnable", (bool)true);
				macroRule.Call ("setRepeatCount", (int)32);
				macroRule.Call ("setEventTimeEnable", (bool)true);
				macroRule.Call ("setEventTimeSamplingCount", (int)12);
				macroRule.Call ("setEventTimeThresholdRange", (int)0, (int)8);
				macroRule.Call ("setDensityEnable", (bool)true);
				macroRule.Call ("setDensityRatio", (int)5);
				
				try {
					mAppGuardClientUnityBridge.Call ("setCustomMacroRule", macroRule);
				}
				catch (NullReferenceException ex) {
					Debug.Log("macroRule was not set");
				}
			}
		#endif
	}

	/**
     * 해당 메소드 호출시, setCustomMacroRule에서 입력된 Macro RuleSet 동작을 시작합니다.
	 *
	 * Macro Customize 정책을 적용할 시점에서 호출하며, 일반적인 호출 방법은 다음과 같습니다.
	 * ex) AppGuardUnityManager.Instance.startCustomMacroMode();
	 *
	 */
	public void startCustomMacroMode() {
		#if UNITY_IPHONE && !UNITY_EDITOR
			AppGuardStartCustomMacroMode();
		#endif
		
		#if UNITY_ANDROID && !UNITY_EDITOR
			if ( mAppGuardClientUnityBridge == null ) {
				mAppGuardClientUnityBridge = new AndroidJavaObject ("com.inca.security.AppGuard.AppGuardClientUnityBridge");
			}

			mAppGuardClientUnityBridge.Call ("startCustomMacroMode");
		#endif
	}

	/**
     * 해당 메소드 호출시, setCustomMacroRule에서 입력된 Macro RuleSet 동작을 중지합니다.
	 *
	 * Macro Customize 정책을 끝낼 시점에 호출하며, 일반적인 호출 방법은 다음과 같습니다.
	 * ex) AppGuardUnityManager.Instance.stopCustomMacroMode();
	 *
	 */
	public void stopCustomMacroMode() {
		#if UNITY_IPHONE && !UNITY_EDITOR
			AppGuardStopCustomMacroMode();
		#endif

		#if UNITY_ANDROID && !UNITY_EDITOR
			if ( mAppGuardClientUnityBridge == null ) {
				mAppGuardClientUnityBridge = new AndroidJavaObject ("com.inca.security.AppGuard.AppGuardClientUnityBridge");
			}

			mAppGuardClientUnityBridge.Call ("stopCustomMacroMode");
		#endif
	}

	
	public static AppGuardUnityManager Instance {
		get {
			if (null == _instance) {
				_instance = FindObjectOfType (typeof(AppGuardUnityManager)) as AppGuardUnityManager;
				if (null == _instance) {
					_instance = new GameObject ("AppGuardUnityManager").AddComponent<AppGuardUnityManager>();

				}
			}
			return _instance;
		}
	}
}