//
//  AppGuard.h
//  AppGuardCore
//
//  Copyright (c) 2016년 INCA Internet Corporation. All rights reserved.
//

#ifndef _APP_GUARD_H_
#define _APP_GUARD_H_

#import <UIKit/UIKit.h>
typedef void (*PAPPGUARDAPPCALLBACK)(int type, int code,  const char *pData);
@protocol AGFrameworkDelegate;

/*
 * AppGuard Framework
 * @public 만 사용
 */

__attribute__((visibility("default"))) @interface AGFramework : NSObject

@property (nonatomic, weak) id<AGFrameworkDelegate>    delegate;
@property (nonatomic)         PAPPGUARDAPPCALLBACK     AppGuardAppCallBack;

+ (AGFramework *)getInt;

/*
 * @private AppGuard에서 사용하는 함수의 이름을 숨기기 위하여 매크로로 처리
 */
#define INITINT                                             CGColorSpaceCopyName
#define INITINT_EX                                          CGColorCreate
#define SET_USER_ID                                         CGColorCreateGenericBGR
#define SENDDELEGATE                                        CGColorCreateGenericBeta
#define DIDFINISHLAUNCHINGWITHOPTIONS                       CGColorCreateGenericGray
#define WILLENTERFOREGROUND                                 CGColorCreateGenericRGB
#define DIDENTERBACKGROUND                                  CGColorCreateGenericCMYK
#define ALERT                                               CGColorCreateCopyWithAlpha
#define SET_CERTIFICATION_ID                                CGColorRetain
#define BUILDPACKET                                         CGColorSpaceCreateIndexed
#define PARSEPACKET                                         CGColorSpaceCreatePattern
#define BUILDJSONPACKET                                     CGColorSpaceCreateWithName
#define PARSEJSONPACKET                                     CGColorSpaceRetain
#define GETUDID                                             CGColorSpaceRelease
#define GETAPPGUARDCALLBACK                                 CGColorSpaceGetTypeID
#define SET_RESERVED2                                       CGColorSpaceGetNumberOfComponents
#define GET_LICENSEKEY                                      CGColorSpaceGetBaseColorSpace
#define START_CUSTOMMACROMODE                               CGColorSpaceGetColorTableCount
#define STOP_CUSTOMMACROMODE                                CGColorSpaceGetColorTable
#define SET_CUSTOMMACRO_REPEAT_ENABLE                       CGColorSpaceCopyICCData
#define SET_CUSTOMMACRO_REPEAT_COUNT                        CGColorSpaceIsWideGamutRGB
#define SET_CUSTOMMACRO_DENSITY_ENABLE                      CGColorSpaceIsHDR
#define SET_CUSTOMMACRO_DENSITY_RATIO                       CGColorSpaceSupportsOutput
#define SET_CUSTOMMACRO_DENSITY_CONDITION_COUNT             CGColorSpaceCopyPropertyList
#define SET_CUSTOMMACRO_DENSITY_CONDITION_ITEM_COUNT        CGColorSpaceCreateWithPropertyList
#define SET_RESERVED1                                       CGColorSpaceCreateWithICCProfile
/*
 * @private AppGuard 내부에서 사용하는 함수
 */
- (void)INITINT:(BOOL)a B:(PAPPGUARDAPPCALLBACK)b;
- (void)INITINT_EX:(BOOL)a B:(id<AGFrameworkDelegate>)b;
- (void)SENDDELEGATE:(int)a B:(int)b C:(const char *)c;
- (void)DIDFINISHLAUNCHINGWITHOPTIONS;
- (void)WILLENTERFOREGROUND;
- (void)DIDENTERBACKGROUND;
- (void)ALERT:(int)a;
- (void)SET_USER_ID:(BOOL)a C:(const char *)c;
- (void)SET_CERTIFICATION_ID:(BOOL)a C:(const char *)c D:(long)d;
- (char*)BUILDPACKET:(BOOL)a B:(const char*)b C:(unsigned int*)c;
- (char*)PARSEPACKET:(BOOL)a B:(const char*)b C:(unsigned int*)c;
- (char*)BUILDJSONPACKET:(BOOL)a C:(const char*)c;
- (char*)PARSEJSONPACKET:(BOOL)a C:(const char*)c;
- (const char*)GETUDID;
- (PAPPGUARDAPPCALLBACK)GETAPPGUARDCALLBACK;
- (void)SET_RESERVED2:(BOOL)a C:(const char*)c;
- (const char*)GET_LICENSEKEY;
- (void)START_CUSTOMMACROMODE;
- (void)STOP_CUSTOMMACROMODE;
- (void)SET_CUSTOMMACRO_REPEAT_ENABLE:(BOOL)a B:(BOOL)b;
- (void)SET_CUSTOMMACRO_REPEAT_COUNT:(BOOL)a B:(int)b;
- (void)SET_CUSTOMMACRO_DENSITY_ENABLE:(BOOL)a B:(BOOL)b;
- (void)SET_CUSTOMMACRO_DENSITY_RATIO:(BOOL)a B:(int)b;
- (void)SET_CUSTOMMACRO_DENSITY_CONDITION_COUNT:(BOOL)a B:(int)b;
- (void)SET_CUSTOMMACRO_DENSITY_CONDITION_ITEM_COUNT:(BOOL)a B:(int)b;
- (void)SET_RESERVED1;
/*
 * #define DISABLE_APPGUARD을 설정하여 AppGuard의 동작을 무시함
 */
#define APPGUARD_TYPE_S2AUTH_CALLBACK    0x9898
#define APPGUARD_TYPE_KILL               0x1818
#define APPGUARD_TYPE_CSAUTH_SUCESS      0x0001
#define APPGUARD_TYPE_CSAUTH_RETRY       0x0002
#define APPGUARD_TYPE_CSAUTH_FALSE       0x0003


#ifdef DISABLE_APPGUARD
    #define APPGUARD_INIT(aDelegate)                                    do { } while(false);
    #define APPGUARD_INIT_EX(aDelegate)                                 do { } while(false);
    #define APPGUARD_SENDDELEGATE(aType, aCode, aData)                  do { } while(false);
    #define APPGUARD_DIDFINISHLAUNCHINGWITHOPTIONS                      do { } while(false);
    #define APPGUARD_WILLENTERFOREGROUND                                do { } while(false);
    #define APPGUARD_DIDENTERBACKGROUND                                 do { } while(false);
    #define APPGUARD_ALERT(a)                                           do { } while(false);
    #define APPGUARD_SET_USER_ID(userId)                                do { } while(false);
    #define APPGUARD_SET_CERTIFICATION_ID(certificationId, aRetry)      do { } while(false);
    #define APPGUARD_BUILDPACKET(packet)                                do { } while(false);
    #define APPGUARD_PARSEPACKET(packet)                                do { } while(false);
    #define APPGUARD_BUILDBASE64PACKET(packet)                          do { } while(false);
    #define APPGUARD_PARSEBASE64PACKET(packet)                          do { } while(false);
    #define APPGUARD_GET_UDID                                           do { } while(false);
    #define APPGUARD_SET_RESERVED2                                      do { } while(false);
    #define APPGUARD_GET_LICENSEKEY                                     do { } while(false);
    #define APPGUARD_START_CUSTOMMACROMODE                              do { } while(false);
    #define APPGUARD_STOP_CUSTOMMACROMODE                               do { } while(false);
    #define APPGUARD_SET_CUSTOMMACRO_REPEAT_ENABLE                      do { } while(false);
    #define APPGUARD_SET_CUSTOMMACRO_REPEAT_COUNT                       do { } while(false);
    #define APPGUARD_SET_CUSTOMMACRO_DENSITY_ENABLE                     do { } while(false);
    #define APPGUARD_SET_CUSTOMMACRO_DENSITY_RATIO                      do { } while(false);
    #define APPGUARD_SET_CUSTOMMACRO_DENSITY_CONDITION_COUNT            do { } while(false);
    #define APPGUARD_SET_CUSTOMMACRO_DENSITY_CONDITION_ITEM_COUNT       do { } while(false);
    #define APPGUARD_SET_RESERVED1                                      do {} while(false);
#else

/*
 * @public AppGuard 초기화 및 실행하는 매크로
 * @param aDelegate 결과를 전달받을 Controller (self)
 * @param aLicenseKey 라이선스 키 (const char *)
 */
    #ifdef DEBUG
        #define APPGUARD_INIT(aDelegate)                                    [[AGFramework getInt] INITINT:YES B:aDelegate];
        #define APPGUARD_INIT_EX(aDelegate)                                 [[AGFramework getInt] INITINT_EX:YES B:aDelegate];
        #define APPGUARD_SET_USER_ID(userId)                                [[AGFramework getInt] SET_USER_ID:YES C:userId];
        #define APPGUARD_SET_CERTIFICATION_ID(certificationId, retry)       [[AGFramework getInt] SET_CERTIFICATION_ID:YES C:certificationId D:retry];
        #define APPGUARD_BUILDPACKET(packet, size)                          [[AGFramework getInt] BUILDPACKET:YES B:packet C:size];
        #define APPGUARD_PARSEPACKET(packet, size)                          [[AGFramework getInt] PARSEPACKET:YES B:packet C:size];
        #define APPGUARD_BUILDBASE64PACKET(packet)                          [[AGFramework getInt] BUILDJSONPACKET:YES C:packet];
        #define APPGUARD_PARSEBASE64PACKET(packet)                          [[AGFramework getInt] PARSEJSONPACKET:YES C:packet];
        #define APPGUARD_SET_RESERVED2(url)                                 [[AGFramework getInt] SET_RESERVED2:YES C:url];
        #define APPGUARD_START_CUSTOMMACROMODE                              [[AGFramework getInt] START_CUSTOMMACROMODE];
        #define APPGUARD_STOP_CUSTOMMACROMODE                               [[AGFramework getInt] STOP_CUSTOMMACROMODE];
        #define APPGUARD_SET_CUSTOMMACRO_REPEAT_ENABLE(enable)              [[AGFramework getInt] SET_CUSTOMMACRO_REPEAT_ENABLE:YES B:enable];
        #define APPGUARD_SET_CUSTOMMACRO_REPEAT_COUNT(repeat)               [[AGFramework getInt] SET_CUSTOMMACRO_REPEAT_COUNT:YES B:repeat];
        #define APPGUARD_SET_CUSTOMMACRO_DENSITY_ENABLE(enable)             [[AGFramework getInt] SET_CUSTOMMACRO_DENSITY_ENABLE:YES B:enable];
        #define APPGUARD_SET_CUSTOMMACRO_DENSITY_RATIO(ratio)               [[AGFramework getInt] SET_CUSTOMMACRO_DENSITY_RATIO:YES B:ratio];
        #define APPGUARD_SET_CUSTOMMACRO_DENSITY_CONDITION_COUNT(count)     [[AGFramework getInt] SET_CUSTOMMACRO_DENSITY_CONDITION_COUNT:YES B:count];
        #define APPGUARD_SET_CUSTOMMACRO_DENSITY_CONDITION_ITEM_COUNT(count)     [[AGFramework getInt] SET_CUSTOMMACRO_DENSITY_CONDITION_ITEM_COUNT:YES B:count];
        #define APPGUARD_SET_RESERVED1                                      [[AGFramework getInt] SET_RESERVED1];
    #else
        #define APPGUARD_INIT(aDelegate)                                    [[AGFramework getInt] INITINT:NO B:aDelegate];
        #define APPGUARD_INIT_EX(aDelegate)                                 [[AGFramework getInt] INITINT_EX:NO B:aDelegate];
        #define APPGUARD_SET_USER_ID(userId)                                [[AGFramework getInt] SET_USER_ID:NO C:userId];
        #define APPGUARD_SET_CERTIFICATION_ID(certificationId, retry)       [[AGFramework getInt] SET_CERTIFICATION_ID:NO C:certificationId D:retry];
        #define APPGUARD_BUILDPACKET(packet, size)                          [[AGFramework getInt] BUILDPACKET:NO B:packet C:size];
        #define APPGUARD_PARSEPACKET(packet, size)                          [[AGFramework getInt] PARSEPACKET:NO B:packet C:size];
        #define APPGUARD_BUILDBASE64PACKET(packet)                          [[AGFramework getInt] BUILDJSONPACKET:NO C:packet];
        #define APPGUARD_PARSEBASE64PACKET(packet)                          [[AGFramework getInt] PARSEJSONPACKET:NO C:packet];
        #define APPGUARD_SET_RESERVED2(url)                                 [[AGFramework getInt] SET_RESERVED2:NO C:url];
        #define APPGUARD_START_CUSTOMMACROMODE                              [[AGFramework getInt] START_CUSTOMMACROMODE];
        #define APPGUARD_STOP_CUSTOMMACROMODE                               [[AGFramework getInt] STOP_CUSTOMMACROMODE];
        #define APPGUARD_SET_CUSTOMMACRO_REPEAT_ENABLE(enable)              [[AGFramework getInt] SET_CUSTOMMACRO_REPEAT_ENABLE:NO B:enable];
        #define APPGUARD_SET_CUSTOMMACRO_REPEAT_COUNT(repeat)               [[AGFramework getInt] SET_CUSTOMMACRO_REPEAT_COUNT:NO B:repeat];
        #define APPGUARD_SET_CUSTOMMACRO_DENSITY_ENABLE(enable)             [[AGFramework getInt] SET_CUSTOMMACRO_DENSITY_ENABLE:NO B:enable];
        #define APPGUARD_SET_CUSTOMMACRO_DENSITY_RATIO(ratio)               [[AGFramework getInt] SET_CUSTOMMACRO_DENSITY_RATIO:NO B:ratio];
        #define APPGUARD_SET_CUSTOMMACRO_DENSITY_CONDITION_COUNT(count)     [[AGFramework getInt] SET_CUSTOMMACRO_DENSITY_CONDITION_COUNT:NO B:count];
        #define APPGUARD_SET_CUSTOMMACRO_DENSITY_CONDITION_ITEM_COUNT(count)     [[AGFramework getInt] SET_CUSTOMMACRO_DENSITY_CONDITION_ITEM_COUNT:NO B:count];
        #define APPGUARD_SET_RESERVED1                                      [[AGFramework getInt] SET_RESERVED1];
    #endif


/*
 * @public 결과를 전달받을 Controller를 설정
 * APPGUARD_INIT을 통하여 설정
 * @param aDelegate 결과를 전달받을 Controller
 */
//#define APPGUARD_SETDELEGATE(aDelegate)                     [[AGFramework getInt] SETDELEGATE:aDelegate];

/*
 * @private AppGuard 내부에서 delegate를 릴레이 해주는 매크로
 * @param aType APPGUARD_TYPE_XXXXX
 * @param aCode APPGUARD_EVENT_XXXXX, APPGUARD_DETECT_XXXXX, APPGUARD_ERROR_XXXXX
 * @param aData 결과에 대한 설명 (const char *)
 */
#define APPGUARD_SENDDELEGATE(aType, aCode, aData)          [[AGFramework getInt] SENDDELEGATE:aType B:aCode C:aData];

/*
 * @public application:didFinishLaunchingWithOptions: 처리
 */
#define APPGUARD_DIDFINISHLAUNCHINGWITHOPTIONS              [[AGFramework getInt] DIDFINISHLAUNCHINGWITHOPTIONS];

/*
 * @public applicationWillEnterForeground: 처리
 */
#define APPGUARD_WILLENTERFOREGROUND                        [[AGFramework getInt] WILLENTERFOREGROUND];

/*
 * @public applicationDidEnterBackground: 처리
 */
#define APPGUARD_DIDENTERBACKGROUND                         [[AGFramework getInt] DIDENTERBACKGROUND];

/*
 * @private AppGuard에서 앱을 강제로 종료할때 호출하는 매크로
 * @param aCode APPGUARD_DETECT_XXXXX를 입력
 */
#define APPGUARD_ALERT(aCode)                               [[AGFramework getInt] ALERT:aCode];

#define APPGUARD_GET_UDID                                   [[AGFramework getInt] GETUDID];
#define APPGUARD_GET_LICENSEKEY                             [[AGFramework getInt] GET_LICENSEKEY]
#endif

@end

/*
 * AppGuard Framework Delegate
 */

@protocol AGFrameworkDelegate <NSObject>

/*
 * @public AppGuard에서 탐지한 결과를 전달
 * @param aFramework AGFramework 객체
 * @param aType APPGUARD_TYPE_XXXXX
 * @param aCode APPGUARD_EVENT_XXXXX, APPGUARD_DETECT_XXXXX, APPGUARD_ERROR_XXXXX
 * @param aData 결과에 대한 설명. 기본은 @"N/A"
 */
@optional
- (void)AGFramework:(AGFramework *)aFramework type:(int)aType code:(int)aCode data:(NSString *)aData;

@end

@interface UIApplication (EventInterceptor)
@end
#endif
